#! /bin/bash -f
#
#  Author: Li-Yan Yuan
#  April 26, 2018, 
#  All rights are reserved.
#
#  This is used to automatically configure and install the RubatoDB server.
#
#  We need to consider the error response when use
#   ssh -l  user_name 10.10.0.2 '/home/rubatodb/RubatoDB0_0/bin/start_rubatodb0_0'
# 
#  (1) Use ssh -l yuan 127.0.0.1 'whoami' to check if the user has access to the IP
#  
#  Note that the initializaiton.sh has been generated with error checking, and this
#  should be used for all other parts.
#

#
#   setup the name of the current working directory
#
DIR=$(
  cd $(dirname "$0")
  pwd
)

cd $DIR

# Display the notice
echo '-------------------------------------------------------------------------'
echo ' '
echo 'Please follow the on-screen instructions, step by step, to install RubatoDB'
echo ' '
echo '  (1)  Pre-installation'
echo '  (2)  Configuration'
echo '  (3)  Check pre-conditions'
echo '  (4)  Installation'
echo '  (5)  Initialization'
echo '  (6)  Start the server'
echo ' '
echo '-------------------------------------------------------------------------'

#
# re-create the distribution directory
#

# Check if the file linux_create.sh exists to avoid repeated creation
if [ -e $DIR/linux_create.sh ]
then
   ./linux_create.sh
   /bin/rm -f linux_create.sh
fi

echo 
echo '(1)  Pre-installation'
echo 
echo '  Please read 3.1.1 Pre-installation of The Documentation of RubatoDB'
echo '  contained in RubatoDB_dist/docs/rubatodb.html'
echo '  Especially, make sure that the following facilities are supported'
echo 
echo '  1.1 Installation and Configuration of Rsync on all servers, including This One'
echo 
echo '      RubatoDB server uses the rsync to communicate among all nodes, and thus'
echo '      the serve will not work at all without rsync installed and configured properly.'
echo 
echo '  1.2 the limit on descriptors shall be at least 16384'
echo 
echo 

#  To make sure the pre-installation has been done properly 

read -p '      Enter Y if the pre-installation 1.1 and 1.2 have been done: ' onevar  
if [ $onevar != "y" ] && [ $onevar != "Y" ]
then 
   echo 
   echo ' Please finish Pre-installation and then come back '
   echo 
   exit
fi

# Ask the user again to make sure that the installation will be done properly
#
echo ' '
read -p '      Are you sure that the pre-installation 1.1 and 1.2 have been done: ' onevar  
if [ $onevar != "y" ] && [ $onevar != "Y" ]
then 
   echo ' '
   echo ' Please finish Pre-installation and then come back '
   echo ' '
   exit
fi

# 
#   Configuration
#
echo ''
echo ''
echo '-------------------------------------------------------------------------'
echo '(2)  Configuration'
echo ' '
echo '  You will be asked in this step to enter the following information, including'
echo '    (2.1) the number of grids '
echo '    (2.2) the number of replication nodes per grid'
echo '    (2.3) the IP addresses and various sockets of each grid node '
echo '    (2.4) the installation directory of each grid node'
echo '    (2.5) the snapshot and log directories of each grid node'
echo '          (at this moment, it must be entered by editing the configuration file)'
echo ' '
echo '  Please note that a RubatoDB server consists of a few grids, and each grid'
echo '  consists of a few replication nodes. For example, you may configure the system with'
echo '  four (4) grid nodes and each grid node will have three (3) replications. In this case'
echo '  you have to specify the followings:'
echo '     (2.1) the number of grids:  4'
echo '     (2.2) the number of replication nodes per grid: 3'
echo '     (2.3) for each 4 x 3 = 12 nodes, specify the IP and sockets'
echo '     (2.4) for each 4 x 3 = 12 nodes, specify the installation directory'
echo '     (2.5) for each 4 x 3 = 12 nodes, specify the snapshot and log directories'
echo ' '
echo '  Please follow the on-screen instructions to enter the requited info for each and every replication node '
echo ' '
#
#  Check if the distribution has been configured earlier
#
cd $DIR

# Check if the file linux_create.sh exists to avoid repeat creation
if [ -e $DIR/bin/start_rubatodb0_0 ]
then
   echo ' The current distribution has been configured earlier'
   echo ''
   read -p ' Enter Y if you want to re-configure it again: ' configure
else
   read -p '  Enter Y if you are going to configure the server now: ' configure 

   if [ "$configure" != "y" ] && [ "$configure" != "Y" ] 
   then 
       echo ' '
       echo '  Goodbye and Welcome Back!'
       echo ' '
       exit
   fi
fi


if [ "$configure" == "y" ] || [ "$configure" == "Y" ] 
then 

    # Check if the file linux_create.sh exists to avoid repeat creation
    #
    if [ -e $DIR/bin/start_rubatodb0_0 ]
    then
	/bin/rm -fr $DIR/bin/start_rubatodb*
    fi
    echo ' '

    read -p '  Enter the user name of the Linux servers: ' username
    while [ "$username" == "root" ]
    do
	echo ' '
	read -p '  One cannot use root for RubatoDB, and please enter it again: ' username
    done

    echo ' '
    echo ' '  Please make sure the RSYNC has been set up properly with user name $username for all nodes to be used

    echo ' '
    read -p '  Enter Y if it is done properly: ' onevar  
    if [ $onevar != "y" ] && [ $onevar != "Y" ]
    then 
	echo ' '
	echo ' Please finish Pre-installation and then come back '
	echo ' '
	exit
    fi

    echo ' '
    echo ' We are going to configure the system now '
    echo ' '
    sleep 2

    cd $DIR/bin
    ./one_step_setup $username $DIR
fi

# 
#   Check all the pre-conditions
#
echo ' '
echo '       Now we are going to check all the pre-conditions on each and every node'
echo ' '


# 
#   Check Pre-conditions
#
echo ' '
echo ' '
echo '-------------------------------------------------------------------------'
echo '(3)  Check Pre-conditions'
echo ' '
echo '       This step will check all the pre-conditions on each and every configured node'
echo ' '

source $DIR/bin/check_pre_condition.sh



# 
#   Installation
#
echo ' '
echo ' '
echo '-------------------------------------------------------------------------'
echo '(4)  Installation'
echo ' '
echo '       This step will install the RubatoDB to all grid nodes, based on the configuration'
echo ' '
echo '       WARNING: all the directories/files in the configured directories will be removed,'
echo '                and we will also kill all RubatoDB processes if any.'
echo ' '

read -p '  Enter Y if you want to install the system: ' install

if [ "$install" == "y" ] || [ "$install" == "Y" ]
then 
    echo ' '
    echo '  We are going to install the system to all the grid nodes '
    echo ' '
    source $DIR/bin/installation.sh
else
    echo ''
    echo ' Goodbye and Welcome Back !'
    echo ''
    exit
fi

# 
#   Initialization'
#
echo ''
echo ''
echo '-------------------------------------------------------------------------'
echo '(5)  Initialization'
echo ' '
echo '       This step will create initial database for each and very node '
echo ' '
echo '       WARNING: all the existing databases/tables in the current system will be removed.'
echo ' '

read -p '  Enter Y if you want to initialize the system: ' initialization

if [ $initialization == 'y' ] || [ $initialization == 'Y' ]
then 
    echo ' '
    echo '  We are going to create the initial system on all the grid nodes '
    echo ' '
    source $DIR/bin/initialization.sh 
else
    echo ''
    echo '  Keep the current system '
    echo ''
fi


# 
#   Start the Server
#
echo ''
echo ''
echo '-------------------------------------------------------------------------'
echo '(6)  Start the Server'
echo ' '
echo '       This will start the server by starting RubatoDB on each and very grid node '
echo '       Please make sure that No other RubatoDB servers are running on the same set of data.'
echo ' '

read -p '  Enter Y if you want to start the server: ' tostart

if [ $tostart == 'y' ] || [ $tostart == 'Y' ]
then 
    echo ' '
    echo '  We are going to start the server, and this may take a few minutes.'
    echo ' '
    source $DIR/bin/start_server.sh 

    echo ' '
    echo ' Well done and Enjoy the RubatoDB server now !'
    echo ' '
    echo ' '
else
    echo ''
    echo ' Goodbye and Welcome Back !'
    echo ''
fi

